\name{ConfigFileOpen}
\alias{ConfigFileOpen}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
%%  ~~function to do ... ~~
}
\description{
This function loads all the data contained in the configuration file specified as parameter 'configfile' in the function Load() of s2dverification.\cr
Extends the environment given as parameter with the variables and functions needed for the configuration file mechanism to work.\cr
This function is called from inside the Load() function. There is no need to call it manually.

Two examples of configuration files can be found inside the 'inst/config/' folder in the package:
  \itemize{
    \item{IC3.conf: configuration file used at IC3. Contains location data on several datasets and variables.}
    \item{template.conf: very simple configuration file intended to be used as pattern when starting from scratch.}
  }
How the configuration file works:\cr
~~~~~~~~~~~~~~~~~~~~~~~~~~~~

It contains two lists and five tables.\cr
Each of these have a header that starts with '!!'. These are key lines and should not be removed or reordered.\cr
Lines starting with '#' and blank lines will be ignored.

The first list should contain all the supported 2-dimensional variables.\cr
The second list should contain all the supported global mean variables.\cr
Regular expressions[1] can be used in these lists to match more than one variable name.

The first table contains information about experiments (which are stored in a file per ensemble per starting date format).\cr
The second table contains information about file-per-member experiments (which are stored in a file per member per starting date format).\cr
The third table contains information about observations (which are stored in a file per month per ensemble format).\cr
The fourth table contains information about the file-per-member observations (which are stored in a file per month per member format).\cr
The fifth table contains information about the file-per-dataset observations (which contain all the dataset values in a single file).

Each table entry is a list of comma-separated elements. \cr
The two first are part of a key that is associated to a value formed by the other elements.\cr
The key elements are a dataset identifier and a variable name.\cr
The value elements are the dataset main path, dataset file path, grid (unless it is an observation table entry), the variable name inside the .nc file, a default suffix (explained below) and a minimum and maximum vaues beyond which loaded data is deactivated.

Given a dataset name and a variable name, a full path is obtained concatenating the main path and the file path with a slash (/) in between.\cr
Also the grid (in the case it is an experiment), the nc variable name, the suffixes and the limit values are obtained.

Any of the elements in the keys can contain regular expressions that will cause matching for sets of dataset names or variable names.

The dataset path and file path can contain shell globbing expressions[2] that will cause matching for sets of paths when fetching the file in the full path.\cr
The full path can point to an OPeNDAP URL.

Any of the elements in the value can contain variables that will be replaced to an associated string.\cr
Variables can be defined only immediately above any of the tables header. \cr
The pattern of a variable definition is\cr
  VARIABLE_NAME = VARIABLE_VALUE\cr
and can be accessed from within the table values or from within the variable values as\cr
  $VARIABLE_NAME$\cr
For example:\cr
  FILE_NAME = tos.nc\cr
  !!table of experiments\cr
  ecmwf, tos, /path/to/dataset, $FILE_NAME$\cr
There are some reserved variables that will offer information about the store frequency, the current startdate Load() is fetching, etc:\cr
  $START_DATE$, $STORE_FREQ$\cr
  for file-per-member datasets: $MEMBER_NUMBER$\cr
  for observations: $YEAR$, $MONTH$, $DAY$\cr
Additionally, from an element in an entry value you can access the other elements of the entry as:\cr
  $EXP_NAME$, $VAR_NAME$, $EXP_MAIN_PATH$, $EXP_FILE_PATH$, $GRID$, $VAR_NAME$, $SUFFIX$, $VAR_MIN$, $VAR_MAX$\cr
and in an observation as:\cr
  $OBS_NAME$, $VAR_NAME$, $OBS_MAIN_PATH$, $OBS_FILE_PATH$, $VAR_NAME$, $SUFFIX$, $VAR_MIN$, $VAR_MAX$

The variable $SUFFIX$ is useful because it can be used to take part in the main or file path. For example: '/path/to$SUFFIX$/dataset'.\cr
It will be replaced by the value in the column that corresponds to the suffix unless the user specifies a different suffix via the parameter 'suffixexp' or 'suffixobs'.\cr
This way the user is able to load two variables with the same name in the same dataset but with slight modifications, with a suffix anywhere in the path to the data that advices of this slight modification. 

The entries in a table will be grouped in 4 levels of specificity:
  \enumerate{
    \item{
General entries:\cr
 · the key dataset name and variable name are both a regular expression matching any sequence of characters (.*) that will cause matching for any pair of dataset and variable names\cr
   Example:  .*, .*, /dataset/main/path, file/path, grid, nc_var_name, suffix, var_min, var_max
    }
    \item{
Dataset entries:\cr
 · the key variable name matches any sequence of characters\cr
   Example:  ecmwf, .*, /dataset/main/path, file/path, grid, nc_var_name, suffix, var_min, var_max
    }
    \item{
Variable entries:\cr
 · the key dataset name matches any sequence of characters\cr
   Example:  .*, tos, /dataset/main/path, file/path, grid, nc_var_name, suffix, var_min, var_max
    }
    \item{
Specific entries:\cr
 · both key values are specified\cr
   Example:  ecmwf, tos, /dataset/main/path, file/path, grid, nc_var_name, suffix, var_min, var_max
    }
  }

Given a pair of dataset name and variable name for which we want to know the full path, all the rules that match will be applied from more general to more specific.\cr
If there is more than one entry per group that match a given key pair, these will be applied in the order of appearance in the configuration file (top to bottom).

An asterisk (*) in any value element will be interpreted as 'leave it as is or take the default value if yet not defined'.\cr
The default values are defined in the following reserved variables:\cr
  $DEFAULT_EXP_MAIN_PATH$, $DEFAULT_EXP_FILE_PATH$, $DEFAULT_GRID$, $DEFAULT_NC_VAR_NAME$, $DEFAULT_OBS_MAIN_PATH$, $DEFAULT_OBS_FILE_PATH$\cr
Trailing asterisks in an entry are not mandatory. For example\cr
  ecmwf, .*, /dataset/main/path, *, *, *, *, *, *\cr
will have the same effect as\cr
  ecmwf, .*, /dataset/main/path

A double quote only (") in any key or value element will be interpreted as 'fill in with the same value as the entry above'.

The pattern to obtain the full path by concatenating the dataset main path and the dataset file path can be adjusted in the variables EXP_FULL_PATH and OBS_FULL_PATH, which are defined by default as\cr
  EXP_FULL_PATH = $EXP_MAIN_PATH$/$EXP_FILE_PATH$\cr
  OBS_FULL_PATH = $OBS_MAIN_PATH$/$OBS_FILE_PATH$\cr

[1] http://tldp.org/LDP/abs/html/globbingref.html\cr
[2] https://stat.ethz.ch/R-manual/R-devel/library/base/html/regex.html
}
\usage{
ConfigFileOpen(file_path, silent = FALSE)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{file_path}{
%%     ~~Describe \code{file_path} here~~
}
  \item{silent}{
%%     ~~Describe \code{silent} here~~
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
%%  ~Describe the value returned
%%  If it is a LIST, use
%%  \item{comp1 }{Description of 'comp1'}
%%  \item{comp2 }{Description of 'comp2'}
%% ...
}
\references{
%% ~put references to the literature/web site here ~
}
\author{
%%  ~~who you are~~
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
##---- Should be DIRECTLY executable !! ----
##-- ==>  Define data, use random,
##--	or do  help(data=index)  for the standard data sets.

## The function is currently defined as
function (file_path, silent = FALSE) 
{
    if (!silent) {
        print(paste("* Reading configuration file:", file_path))
    }
    all_lines <- readLines(file_path)
    all_lines <- gsub("\t", "", all_lines)
    all_lines <- gsub(" ", "", all_lines)
    all_lines <- all_lines[-grep("^#", all_lines)]
    all_lines <- all_lines[-grep("^$", all_lines)]
    key_positions <- grep("^!!", all_lines)
    if (length(key_positions) != 12) {
        stop("Error: The configuration file is corrupted or outdated: the key lines do not match the expected pattern.")
    }
    if (key_positions[1] + 1 < key_positions[2]) {
        two_d_var_lines <- all_lines[(key_positions[1] + 1):(key_positions[2] - 
            1)]
        two_d_vars <- unlist(strsplit(two_d_var_lines, ","))
    }
    else {
        two_d_vars <- c()
    }
    if (key_positions[2] + 1 < key_positions[3]) {
        global_mean_var_lines <- all_lines[(key_positions[2] + 
            1):(key_positions[3] - 1)]
        global_mean_vars <- unlist(strsplit(gsub(" ", "", global_mean_var_lines), 
            ","))
    }
    else {
        global_mean_vars <- c()
    }
    definitions <- c()
    if (key_positions[3] + 1 < key_positions[4]) {
        exp_table_definitions <- all_lines[(key_positions[3] + 
            1):(key_positions[4] - 1)]
    }
    else {
        exp_table_definitions <- c()
    }
    if (key_positions[5] + 1 < key_positions[6]) {
        file_per_member_exp_table_definitions <- all_lines[(key_positions[5] + 
            1):(key_positions[6] - 1)]
    }
    else {
        file_per_member_exp_table_definitions <- c()
    }
    if (key_positions[7] + 1 < key_positions[8]) {
        obs_table_definitions <- all_lines[(key_positions[7] + 
            1):(key_positions[8] - 1)]
    }
    else {
        obs_table_definitions <- c()
    }
    if (key_positions[9] + 1 < key_positions[10]) {
        file_per_member_obs_table_definitions <- all_lines[(key_positions[9] + 
            1):(key_positions[10] - 1)]
    }
    else {
        file_per_member_obs_table_definitions <- c()
    }
    if (key_positions[11] + 1 < key_positions[12]) {
        file_per_dataset_obs_table_definitions <- all_lines[(key_positions[11] + 
            1):(key_positions[12] - 1)]
    }
    else {
        file_per_dataset_obs_table_definitions <- c()
    }
    all_definitions <- c(exp_table_definitions, file_per_member_exp_table_definitions, 
        obs_table_definitions)
    all_definitions <- c(all_definitions, file_per_member_obs_table_definitions, 
        file_per_dataset_obs_table_definitions)
    if (length(grep("=", all_definitions)) == length(all_definitions)) {
        for (definition in all_definitions) {
            if (length(which(strsplit(definition, "")[[1]] == 
                "=")) == 1) {
                var_name <- strsplit(definition, "=")[[1]][1]
                var_value <- strsplit(definition, "=")[[1]][2]
                if ((length(which(strsplit(var_value, "")[[1]] == 
                  "$"))\%\%2) == 0) {
                  definitions <- ConfigPutInDictionary(definitions, 
                    var_name, var_value)
                }
                else {
                  stop("Error: The configuration file is corrupted: there are incorrect variable definition lines in the definition zones. A closing \"$\" symbol may be missing.")
                }
            }
            else {
                stop("Error: The configuration file is corrupted: there are incorrect definition lines in the definition zones.")
            }
        }
    }
    else {
        stop("Error: The configuration file is corrupted: there are malformed definition lines in the definition zones.")
    }
    tables_key_positions <- c(4, 6, 8, 10, 12)
    current_table <- 1
    for (table_key_position in tables_key_positions) {
        datasets <- list(c(), c(), c(), c())
        if (current_table < 3) {
            default_values <- c("$DEFAULT_EXP_MAIN_PATH$", "$DEFAULT_EXP_FILE_PATH$", 
                "$DEFAULT_GRID$", "$DEFAULT_NC_VAR_NAME$", "$DEFAULT_SUFFIX$", 
                "$DEFAULT_VAR_MIN$", "$DEFAULT_VAR_MAX$")
        }
        else {
            default_values <- c("$DEFAULT_OBS_MAIN_PATH$", "$DEFAULT_OBS_FILE_PATH$", 
                "$DEFAULT_NC_VAR_NAME$", "$DEFAULT_SUFFIX$", 
                "$DEFAULT_VAR_MIN$", "$DEFAULT_VAR_MAX$")
        }
        previous_values <- c(".*", ".*", default_values)
        table_lines <- c()
        table_end <- ifelse(table_key_position == max(tables_key_positions), 
            length(all_lines), key_positions[table_key_position + 
                1] - 1)
        if ((key_positions[table_key_position] + 1) <= table_end) {
            table_lines <- all_lines[(key_positions[table_key_position] + 
                1):table_end]
            table_lines <- strsplit(table_lines, ",")
        }
        for (entry in table_lines) {
            if (entry[1] == "\"") {
                entry[1] <- previous_values[1]
            }
            if ((length(entry) > 1)) {
                if (entry[2] == "\"") {
                  entry[2] <- previous_values[2]
                }
            }
            else {
                stop("Error: The variable column must be defined in all the entries in the tables in the configuration file.")
            }
            for (value_position in 1:length(default_values)) {
                if ((length(entry) > value_position + 1)) {
                  if (entry[value_position + 2] == "\"") {
                    entry[value_position + 2] <- previous_values[value_position + 
                      2]
                  }
                }
                else {
                  entry[value_position + 2] <- "*"
                }
            }
            if (entry[1] == ".*") {
                if (entry[2] == ".*") {
                  datasets[[1]] <- c(datasets[[1]], list(entry))
                }
                else {
                  datasets[[3]] <- c(datasets[[3]], list(entry))
                }
            }
            else {
                if (entry[2] == ".*") {
                  datasets[[2]] <- c(datasets[[2]], list(entry))
                }
                else {
                  datasets[[4]] <- c(datasets[[4]], list(entry))
                }
            }
            previous_values <- entry
        }
        if (current_table == 1) {
            file_per_startdate_exps <- datasets
        }
        else if (current_table == 2) {
            file_per_member_exps <- datasets
        }
        else if (current_table == 3) {
            file_per_month_obs <- datasets
        }
        else if (current_table == 4) {
            file_per_member_obs <- datasets
        }
        else if (current_table == 5) {
            file_per_dataset_obs <- datasets
        }
        current_table <- current_table + 1
    }
    if (!silent) {
        print("* Config file read successfully.")
    }
    invisible(list(definitions = definitions, two_d_vars = two_d_vars, 
        global_mean_vars = global_mean_vars, file_per_startdate_experiments = file_per_startdate_exps, 
        file_per_member_experiments = file_per_member_exps, file_per_month_observations = file_per_month_obs, 
        file_per_member_observations = file_per_member_obs, file_per_dataset_observations = file_per_dataset_obs))
  }
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ ~kwd1 }
\keyword{ ~kwd2 }% __ONLY ONE__ keyword per line
